package helper

import (
	"fmt"
	"os"

	"github.com/labstack/echo/v4"
	"github.com/onsi/gomega"
	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"
)

// RouteExists searches through a slice of echo routes for a specific echo route.
//
// Matches on method and path.
func RouteExists(routes []*echo.Route, target echo.Route) bool {
	for _, route := range routes {
		if route.Method == target.Method &&
			route.Path == target.Path {
			return true
		}
	}

	return false
}

// ExpectRoutes takes an array of routes to search through for the given expected routes.
//
// The expectedRoutes parameter is an array of string pairs (<METHOD>, <PATH>).
func ExpectRoutes(routes []*echo.Route, expectedRoutes []string) {
	logger := log.Output(zerolog.NewConsoleWriter()).With().Logger()
	logger.Debug().Msgf("looking for routes '%v' in '%v'", expectedRoutes, routes)

	for i := 0; i < len(expectedRoutes); i += 2 {
		gomega.Expect(RouteExists(routes, echo.Route{
			Method: expectedRoutes[i],
			Path:   expectedRoutes[i+1],
			Name:   "",
		})).To(gomega.BeTrue())
	}
}

func createConfigFile() *os.File {
	configFile, err := os.Create("config.yaml")
	if err != nil {
		log.Err(err).Msg("error: failed to generate config file")
	}

	return configFile
}

func GenerateConfig(yaml string) {
	configFile := createConfigFile()
	defer configFile.Close()

	bytesWritten, err := configFile.WriteString(yaml)
	if err != nil {
		log.Err(err).Msg("error: failed to write to config file")
	}

	if bytesWritten != len(yaml) {
		log.Err(err).Msg("error: partial write to config file")
	}
}

// GenerateBadConfig creates a bad yaml config file.
func GenerateBadConfig() {
	GenerateConfig("%garbage")
}

// GenerateGoodConfig creates a good yaml config file.
func GenerateGoodConfig() {
	GenerateConfig("app.name: test")
}

// CleanupConfig removes the yaml config file.
func CleanupConfig() {
	if err := os.Remove("config.yaml"); err != nil {
		log.Err(err).Msg("error: failed to cleanup config file")
	}
}

// NonFatalLogger wraps the zerolog logger and overrides the Fatal method to only log at Error level.
//
// Useful for testing code that calls Fatal.
type NonFatalLogger struct {
	*zerolog.Logger

	FatalCallsCount int
}

// Fatal does not call os.Exit so is not actually fatal. Use only for testing.
func (logger *NonFatalLogger) Fatal() *zerolog.Event {
	logger.FatalCallsCount++

	return logger.Panic()
}

// NewNonFatalLogger creates a new NonFatalLogger with the global zerolog logger.
func NewNonFatalLogger() *NonFatalLogger {
	return &NonFatalLogger{
		Logger:          &log.Logger,
		FatalCallsCount: 0,
	}
}

func GenerateWriterDoStore(store *string) func(interface{}) int {
	return func(i interface{}) int {
		str := fmt.Sprintf("%s", i)
		*store = str

		return len(str)
	}
}
